# doublearm.py
#

print("loading doublearm.py...")

# Import the Webots simulator API.
from controller import Robot
from controller import Keyboard

# Import the standard Python math library.
import math

# Define the time step in milliseconds between
# controller updates.
EVENT_LOOP_DT = 20

################################################################

# Request a proxy object representing the robot to control.
robot = Robot()
name = robot.getName()
print(f"doublearm.py waking up for {name}...")

# Enable computer keyboard input for user control.
keyboard = Keyboard()
keyboard.enable(EVENT_LOOP_DT)

# Fetch handle for the 'base' joint motor.  In this
# example the motor will be controlled as a torque
# motor, bypassing the Webots PID control.
motor1 = robot.getDevice('rmR1')
motor1.setTorque(0.0)

motor2 = robot.getDevice('rmL1')
motor2.setTorque(0.0)

motor3 = robot.getDevice('rmR2')
motor3.setTorque(0.0)

motor4 = robot.getDevice('rmL2')
motor4.setTorque(0.0)

# set properties in RotationalMotor nodes:
# motor1.setAvailableTorque(4)
# motor2.setAvailableTorque(4)
# motor3.setAvailableTorque(2)
# motor4.setAvailableTorque(2)

# Specify proportional and derivative (damping) gains
# for the position controllers in the main event loop.
P_gain1 = 8.0     # units are N-m / radian
D_gain1 = 0.2     # units are N-m / (radian/sec)

P_gain2 = 4.0     # units are N-m / radian
D_gain2 = 0.1     # units are N-m / (radian/sec)

# Soft torque limits to avoid error messages.
max_tau1 = 4.0  # units are N-m
max_tau2 = 1.0  # units are N-m

# Initial position targets.
q_d1 = None
q_d2 = None

# The controller estimates velocities using finite
# differencing of the position sensors; these variables
# hold the previous state.
last_q1 = 0
last_q2 = 0

################################################################
# Define reference poses mapped to keyboard keys.

poses = {
    'w' : (0.0,  0.0),          # straight up
    'd' : (0.5, -1.0),          # elbow to machine right (-Y)
    'a' : (-0.5, 1.0),          # elbow to machine left (+Y)
    's' : (math.pi, -math.pi),  # elbow at bottom, link2 pointing up
    'q' : (2*math.pi, 0.0),     # straight up, but wound a full positive rotation
}

# Set joint position targets to a reference pose indicated by a single character string.
def set_pose(key):
    global q_d1, q_d2
    # special case: 'p' will enter a passive zero-torque mode
    if key == 'p':
        q_d1 = None
        q_d2 = None
    else:
        # look for a reference pose
        pose = poses.get(key)
        if pose is not None:
            q_d1 = pose[0]
            q_d2 = pose[1]

################################################################

# Run an event loop until the simulation quits,
# indicated by the step function returning -1.
while robot.step(EVENT_LOOP_DT) != -1:

    # Read simulator clock time.
    # t = robot.getTime()

    # Read the new joint positions.
    # q1 = j1.getValue()
    # q2 = j2.getValue()

    # Estimate current velocities in radians/sec using finite differences.
    # qd1 = (q1 - last_q1) / (0.001 * EVENT_LOOP_DT)
    # qd2 = (q2 - last_q2) / (0.001 * EVENT_LOOP_DT)
    # last_q1 = q1
    # last_q2 = q2

    # Read any computer keyboard keypresses.  Returns -1 or an integer keycode while a key is held down.
    # if name == 'left':
        # key = keyboard.getKey()
        # if key != -1:
            # convert the integer key number to a lowercase single-character string,
            # then set the target pose indicated by the key
            # set_pose(chr(key).lower())

    # else:
        # the other devices run an automatic timed sequence
        # set_automatic_targets(t)

    # Calculate new motor torques, limit them, and apply them to the system.
    # if q_d1 is None:
        # tau1 = 0.0
    # else:
        # tau1 = P_gain1 * (q_d1 - q1) - D_gain1 * qd1
        # tau1 = min(max(tau1, -max_tau1), max_tau1)

    # if q_d2 is None:
        # tau2 = 0.0
    # else:
        # tau2 = P_gain2 * (q_d2 - q2) - D_gain2 * qd2
        # tau2 = min(max(tau2, -max_tau2), max_tau2)

    key = keyboard.getKey()
    if key != -1:
        kref = ord(chr(key).lower())
        print(kref)
        if kref == 102:
            motor1.setTorque(10)
        elif kref == 100:
            motor1.setTorque(-10)
        
        if kref == 106:
            motor2.setTorque(10)
        elif kref == 107:
            motor2.setTorque(-10)
            
        if kref == 101:
            motor3.setTorque(6)
        elif kref == 114:
            motor3.setTorque(-6)
        
        if kref == 117:
            motor4.setTorque(6)
        elif kref == 105:
            motor4.setTorque(-6)

    motor1.setTorque(10)
    motor2.setTorque(10)
